//
// Created by huli on 2021/3/24.
//

#include "dispatch_coordinates.h"
#include "../tool/proto_tool.h"


Dispatch_coordinates::Dispatch_coordinates()
{
	mpp_space_lock = nullptr;
	m_space_lock_rows = 0;
	m_space_lock_columns = 0;
	m_space_lock_total = 0;

}
Dispatch_coordinates::~Dispatch_coordinates()
{
	dispatch_coordinates_uninit();
}

//初始化 调度的坐标系 模块。如下三选一
Error_manager Dispatch_coordinates::dispatch_coordinates_init()
{
	return  dispatch_coordinates_init_from_protobuf(DISPATCH_COORDINATES_PARAMETER_PATH);
}

//初始化 调度的坐标系 模块。从文件读取
Error_manager Dispatch_coordinates::dispatch_coordinates_init_from_protobuf(std::string prototxt_path)
{
	Dispatch_proto::Dispatch_coordinates_parameter_all t_dispatch_coordinates_parameter_all;
	if(!  proto_tool::read_proto_param(prototxt_path,t_dispatch_coordinates_parameter_all) )
	{
		return Error_manager(DISPATCH_COORDINATES_READ_PROTOBUF_ERROR,MINOR_ERROR,
							 "Dispatch_coordinates read_proto_param  failed");
	}

	return dispatch_coordinates_init_from_protobuf(t_dispatch_coordinates_parameter_all);
}

//初始化 调度的坐标系 模块。从protobuf读取
Error_manager Dispatch_coordinates::dispatch_coordinates_init_from_protobuf(Dispatch_proto::Dispatch_coordinates_parameter_all& dispatch_coordinates_parameter_all)
{
	LOG(INFO) << " ---Dispatch_coordinates::dispatch_coordinates_init_from_protobuf() run--- "<< this;
	Error_manager t_error;

	m_space_lock_rows = dispatch_coordinates_parameter_all.parkspace_number().rows();
	m_space_lock_columns = dispatch_coordinates_parameter_all.parkspace_number().columns();
	m_space_lock_total = dispatch_coordinates_parameter_all.parkspace_number().total();
	mpp_space_lock = new space_device*[m_space_lock_rows];
	for (int i = 0; i < m_space_lock_rows; ++i)
	{
		mpp_space_lock[i] = new space_device[m_space_lock_columns];
		for (int j = 0; j < m_space_lock_columns; ++j)
		{
			//注意了, 这里一定要初始化为-1, 因为设备id=0 是有效设备
			mpp_space_lock[i][j].m_catcher_id = -1;
			mpp_space_lock[i][j].m_carrier_id = -1;
		}
	}



	int size;
	size = dispatch_coordinates_parameter_all.packspace_coordinates_parameters_size();
	for (int i = 0; i < size; ++i)
	{
		Point3D_tool::Point3D t_point3d;
		int id = dispatch_coordinates_parameter_all.packspace_coordinates_parameters(i).id();
		t_point3d.x = dispatch_coordinates_parameter_all.packspace_coordinates_parameters(i).x();
		t_point3d.y = dispatch_coordinates_parameter_all.packspace_coordinates_parameters(i).y();
		t_point3d.z = dispatch_coordinates_parameter_all.packspace_coordinates_parameters(i).z();
		m_packspace_coordinates[id] = t_point3d;
	}

	size = dispatch_coordinates_parameter_all.passageway_coordinates_parameters_size();
	for (int i = 0; i < size; ++i)
	{
		Point3D_tool::Point3D t_point3d;
		int id = dispatch_coordinates_parameter_all.passageway_coordinates_parameters(i).id();
		t_point3d.x = dispatch_coordinates_parameter_all.passageway_coordinates_parameters(i).x();
		t_point3d.y = dispatch_coordinates_parameter_all.passageway_coordinates_parameters(i).y();
		t_point3d.z = dispatch_coordinates_parameter_all.passageway_coordinates_parameters(i).z();
		m_passageway_coordinates[id] = t_point3d;
	}

	size = dispatch_coordinates_parameter_all.carrier_coordinates_parameters_size();
	for (int i = 0; i < size; ++i)
	{
		Point3D_tool::Point3D t_point3d;
		int id = dispatch_coordinates_parameter_all.carrier_coordinates_parameters(i).id();
		t_point3d.x = dispatch_coordinates_parameter_all.carrier_coordinates_parameters(i).x();
		t_point3d.y = dispatch_coordinates_parameter_all.carrier_coordinates_parameters(i).y();
		t_point3d.z = dispatch_coordinates_parameter_all.carrier_coordinates_parameters(i).z();
		m_carrier_coordinates[id] = t_point3d;
	}

	size = dispatch_coordinates_parameter_all.catcher_coordinates_parameters_size();
	for (int i = 0; i < size; ++i)
	{
		Point3D_tool::Point3D t_point3d;
		int id = dispatch_coordinates_parameter_all.catcher_coordinates_parameters(i).id();
		t_point3d.x = dispatch_coordinates_parameter_all.catcher_coordinates_parameters(i).x();
		t_point3d.y = dispatch_coordinates_parameter_all.catcher_coordinates_parameters(i).y();
		t_point3d.z = dispatch_coordinates_parameter_all.catcher_coordinates_parameters(i).z();
		m_catcher_coordinates[id] = t_point3d;
	}


	m_carrier_box.x_min = dispatch_coordinates_parameter_all.carrier_box_parameters().x_min();
	m_carrier_box.x_max = dispatch_coordinates_parameter_all.carrier_box_parameters().x_max();
	m_carrier_box.y_min = dispatch_coordinates_parameter_all.carrier_box_parameters().y_min();
	m_carrier_box.y_max = dispatch_coordinates_parameter_all.carrier_box_parameters().y_max();
	m_carrier_box.z_min = dispatch_coordinates_parameter_all.carrier_box_parameters().z_min();
	m_carrier_box.z_max = dispatch_coordinates_parameter_all.carrier_box_parameters().z_max();

	m_catcher_box.x_min = dispatch_coordinates_parameter_all.catcher_box_parameters().x_min();
	m_catcher_box.x_max = dispatch_coordinates_parameter_all.catcher_box_parameters().x_max();
	m_catcher_box.y_min = dispatch_coordinates_parameter_all.catcher_box_parameters().y_min();
	m_catcher_box.y_max = dispatch_coordinates_parameter_all.catcher_box_parameters().y_max();
	m_catcher_box.z_min = dispatch_coordinates_parameter_all.catcher_box_parameters().z_min();
	m_catcher_box.z_max = dispatch_coordinates_parameter_all.catcher_box_parameters().z_max();

	m_catcher_b_min = dispatch_coordinates_parameter_all.catcher_b_min();
	m_catcher_b_max = dispatch_coordinates_parameter_all.catcher_b_max();
	m_catcher_d1_min = dispatch_coordinates_parameter_all.catcher_d1_min();
	m_catcher_d1_max = dispatch_coordinates_parameter_all.catcher_d1_max();	
	m_catcher_d2_min = dispatch_coordinates_parameter_all.catcher_d2_min();
	m_catcher_d2_max = dispatch_coordinates_parameter_all.catcher_d2_max();

	std::cout << " huli test :::: " << " m_packspace_coordinates.size() = " << m_packspace_coordinates.size() << std::endl;
	std::cout << " huli test :::: " << " m_passageway_coordinates.size() = " << m_passageway_coordinates.size() << std::endl;
	std::cout << " huli test :::: " << " m_carrier_coordinates.size() = " << m_carrier_coordinates.size() << std::endl;
	std::cout << " huli test :::: " << " m_catcher_coordinates.size() = " << m_catcher_coordinates.size() << std::endl;

	m_default_wheelbase = dispatch_coordinates_parameter_all.default_wheelbase();

	m_catcher_1th_floor_z = dispatch_coordinates_parameter_all.catcher_1th_floor_z();
	m_catcher_2th_floor_z = dispatch_coordinates_parameter_all.catcher_2th_floor_z();
	m_catcher_3th_floor_z = dispatch_coordinates_parameter_all.catcher_3th_floor_z();
	m_catcher_4th_floor_z = dispatch_coordinates_parameter_all.catcher_4th_floor_z();

	m_carrier_1th_floor_z = dispatch_coordinates_parameter_all.carrier_1th_floor_z();
	m_carrier_2th_floor_z = dispatch_coordinates_parameter_all.carrier_2th_floor_z();
	m_carrier_3th_floor_z = dispatch_coordinates_parameter_all.carrier_3th_floor_z();
	m_carrier_4th_floor_z = dispatch_coordinates_parameter_all.carrier_4th_floor_z();

	m_catcher_d1_d2_distance = dispatch_coordinates_parameter_all.catcher_d1_d2_distance();
	m_catcher_wheel_base_limit = dispatch_coordinates_parameter_all.catcher_wheel_base_limit();

	m_carrier_y_y1_distance = dispatch_coordinates_parameter_all.carrier_y_y1_distance();
	m_carrier_default_y1_back = dispatch_coordinates_parameter_all.carrier_default_y1_back();
	m_carrier_default_y1_leave = dispatch_coordinates_parameter_all.carrier_default_y1_leave();
	m_carrier_default_y_back = dispatch_coordinates_parameter_all.carrier_default_y_back();
	m_carrier_default_y_leave = dispatch_coordinates_parameter_all.carrier_default_y_leave();

	m_carrier_default_x_left = dispatch_coordinates_parameter_all.carrier_default_x_left();
	m_carrier_default_x_right = dispatch_coordinates_parameter_all.carrier_default_x_right();

	m_passageway_terminal_id_min = dispatch_coordinates_parameter_all.passageway_terminal_id_min();
	m_passageway_terminal_id_max = dispatch_coordinates_parameter_all.passageway_terminal_id_max();
	m_passageway_terminal_number = dispatch_coordinates_parameter_all.passageway_terminal_number();
	size = dispatch_coordinates_parameter_all.passageway_functioning_patterns_size();
	for (int i = m_passageway_terminal_id_min; i < size; ++i)
	{
		m_passageway_functioning_pattern_map[i] = (Passageway_functioning_pattern)dispatch_coordinates_parameter_all.passageway_functioning_patterns(i);
	}
	if ( m_passageway_terminal_number != m_passageway_terminal_id_max - m_passageway_terminal_id_min +1 ||
		 m_passageway_terminal_number != size)
	{
		return Error_manager(Error_code::PARAMETER_ERROR, Error_level::MINOR_ERROR,
							 " Dispatch_coordinates::dispatch_coordinates_init_from_protobuf PARAMRTER ERROR ");
	}

	m_parkspace_id_min = dispatch_coordinates_parameter_all.parkspace_id_min();
	m_parkspace_id_max = dispatch_coordinates_parameter_all.parkspace_id_max();
	m_parkspace_id_total = dispatch_coordinates_parameter_all.parkspace_id_total();

	m_car_wheel_base_min = dispatch_coordinates_parameter_all.car_wheel_base_min();
	m_car_wheel_base_max = dispatch_coordinates_parameter_all.car_wheel_base_max();

	return Error_code::SUCCESS;
}
//反初始化
Error_manager Dispatch_coordinates::dispatch_coordinates_uninit()
{
	for (int i = 0; i < m_space_lock_rows; ++i)
	{
		delete [](mpp_space_lock[i]);
	}
	delete [](mpp_space_lock);
	return Error_code::SUCCESS;
}


void Dispatch_coordinates::set_space_lock(int space_rows, int space_columns, int catcher_id, int carrier_id)
{
	std::unique_lock<std::mutex> t_lock(m_lock);
	mpp_space_lock[space_rows][space_columns].m_catcher_id = catcher_id;
	mpp_space_lock[space_rows][space_columns].m_carrier_id = carrier_id;
}
void Dispatch_coordinates::set_space_lock_for_catcher(int space_rows, int space_columns, int catcher_id)
{
	std::unique_lock<std::mutex> t_lock(m_lock);
	mpp_space_lock[space_rows][space_columns].m_catcher_id = catcher_id;
}
void Dispatch_coordinates::set_space_lock_for_carrier(int space_rows, int space_columns, int carrier_id)
{
	std::unique_lock<std::mutex> t_lock(m_lock);
	mpp_space_lock[space_rows][space_columns].m_carrier_id = carrier_id;
}
void Dispatch_coordinates::set_space_lock(int space_id, int catcher_id, int carrier_id)
{
	int space_rows = (space_id-1) / m_space_lock_columns;
	int space_columns = (space_id-1) % m_space_lock_columns;
	set_space_lock(space_rows, space_columns, catcher_id, carrier_id);
}
void Dispatch_coordinates::set_space_lock_for_catcher(int space_id, int catcher_id)
{
	int space_rows = (space_id-1) / m_space_lock_columns;
	int space_columns = (space_id-1) % m_space_lock_columns;
	set_space_lock_for_catcher(space_rows, space_columns, catcher_id);
}
void Dispatch_coordinates::set_space_lock_for_carrier(int space_id, int carrier_id)
{
	int space_rows = (space_id-1) / m_space_lock_columns;
	int space_columns = (space_id-1) % m_space_lock_columns;
	set_space_lock_for_carrier(space_rows, space_columns, carrier_id);
}

